import { Authentication } from "./Authentication";

export class FileManager {
  private auth: Authentication;
  private static instance: FileManager;

  private constructor() {
    this.auth = Authentication.getInstance();
  }

  /**
   * Get the singleton instance of FileManager
   */
  public static getInstance(): FileManager {
    if (!FileManager.instance) {
      FileManager.instance = new FileManager();
    }
    return FileManager.instance;
  }

  /**
   * Upload a single file to a record
   * @param collectionName The name of the collection
   * @param recordId The ID of the record to attach the file to
   * @param field The field name for the file
   * @param file The file to upload
   * @returns The updated record
   */
  public async uploadFile<T = any>(
    collectionName: string,
    recordId: string,
    field: string,
    file: File
  ): Promise<T> {
    if (!this.auth.isAuthenticated()) {
      throw new Error("User must be authenticated to upload files");
    }

    try {
      this.auth.setUpdating(true);
      const pb = this.auth.getPocketBase();
      const formData = new FormData();
      formData.append(field, file);

      const result = await pb.collection(collectionName).update<T>(recordId, formData);
      return result;
    } catch (err) {
      console.error(`Failed to upload file to ${collectionName}:`, err);
      throw err;
    } finally {
      this.auth.setUpdating(false);
    }
  }

  /**
   * Upload multiple files to a record with chunked upload support
   * @param collectionName The name of the collection
   * @param recordId The ID of the record to attach the files to
   * @param field The field name for the files
   * @param files Array of files to upload
   * @returns The updated record
   */
  public async uploadFiles<T = any>(
    collectionName: string,
    recordId: string,
    field: string,
    files: File[]
  ): Promise<T> {
    if (!this.auth.isAuthenticated()) {
      throw new Error("User must be authenticated to upload files");
    }

    try {
      this.auth.setUpdating(true);
      const pb = this.auth.getPocketBase();
      
      const MAX_FILE_SIZE = 50 * 1024 * 1024; // 50MB per file limit
      const MAX_BATCH_SIZE = 25 * 1024 * 1024; // 25MB per batch
      
      // Validate file sizes first
      for (const file of files) {
        if (file.size > MAX_FILE_SIZE) {
          throw new Error(`File ${file.name} is too large. Maximum size is 50MB.`);
        }
      }

      // Get existing record if updating
      let existingFiles: string[] = [];
      if (recordId) {
        try {
          const record = await pb.collection(collectionName).getOne<T>(recordId);
          existingFiles = (record as any)[field] || [];
        } catch (error) {
          console.warn('Failed to fetch existing record:', error);
        }
      }

      // Process files in batches
      let currentBatchSize = 0;
      let currentBatch: File[] = [];
      let allProcessedFiles: File[] = [];

      // Process each file
      for (const file of files) {
        let processedFile = file;
        
        try {
          // Try to compress image files if needed
          if (file.type.startsWith('image/')) {
            processedFile = await this.compressImageIfNeeded(file, 50); // 50MB max size
          }
        } catch (error) {
          console.warn(`Failed to process file ${file.name}:`, error);
          processedFile = file; // Use original file if processing fails
        }

        // Check if adding this file would exceed batch size
        if (currentBatchSize + processedFile.size > MAX_BATCH_SIZE) {
          // Upload current batch
          if (currentBatch.length > 0) {
            await this.uploadBatch(collectionName, recordId, field, currentBatch);
            allProcessedFiles.push(...currentBatch);
          }
          // Reset batch
          currentBatch = [processedFile];
          currentBatchSize = processedFile.size;
        } else {
          // Add to current batch
          currentBatch.push(processedFile);
          currentBatchSize += processedFile.size;
        }
      }

      // Upload any remaining files
      if (currentBatch.length > 0) {
        await this.uploadBatch(collectionName, recordId, field, currentBatch);
        allProcessedFiles.push(...currentBatch);
      }

      // Get the final record state
      const finalRecord = await pb.collection(collectionName).getOne<T>(recordId);
      return finalRecord;
    } catch (err) {
      console.error(`Failed to upload files to ${collectionName}:`, err);
      throw err;
    } finally {
      this.auth.setUpdating(false);
    }
  }

  /**
   * Upload a batch of files
   * @private
   */
  private async uploadBatch<T = any>(
    collectionName: string,
    recordId: string,
    field: string,
    files: File[]
  ): Promise<void> {
    const pb = this.auth.getPocketBase();
    const formData = new FormData();

    // Add new files
    for (const file of files) {
      formData.append(field, file);
    }

    try {
      await pb.collection(collectionName).update(recordId, formData);
    } catch (error: any) {
      if (error.status === 413) {
        throw new Error(`Upload failed: Batch size too large. Please try uploading smaller files.`);
      }
      throw error;
    }
  }

  /**
   * Append multiple files to a record without overriding existing ones
   * @param collectionName The name of the collection
   * @param recordId The ID of the record to attach the files to
   * @param field The field name for the files
   * @param files Array of files to upload
   * @returns The updated record
   */
  public async appendFiles<T = any>(
    collectionName: string,
    recordId: string,
    field: string,
    files: File[]
  ): Promise<T> {
    if (!this.auth.isAuthenticated()) {
      throw new Error("User must be authenticated to upload files");
    }

    try {
      this.auth.setUpdating(true);
      const pb = this.auth.getPocketBase();
      
      // First, get the current record to check existing files
      const record = await pb.collection(collectionName).getOne<T>(recordId);
      
      // Create FormData with existing files
      const formData = new FormData();

      // Get existing files from the record
      const existingFiles = (record as any)[field] || [];

      // For each existing file, we need to fetch it and add it to the FormData
      for (const existingFile of existingFiles) {
        try {
          const response = await fetch(this.getFileUrl(collectionName, recordId, existingFile));
          const blob = await response.blob();
          const file = new File([blob], existingFile, { type: blob.type });
          formData.append(field, file);
        } catch (error) {
          console.warn(`Failed to fetch existing file ${existingFile}:`, error);
        }
      }
      
      // Append new files
      files.forEach(file => {
        formData.append(field, file);
      });

      const result = await pb.collection(collectionName).update<T>(recordId, formData);
      return result;
    } catch (err) {
      console.error(`Failed to append files to ${collectionName}:`, err);
      throw err;
    } finally {
      this.auth.setUpdating(false);
    }
  }

  /**
   * Get the URL for a file
   * @param collectionName The name of the collection
   * @param recordId The ID of the record containing the file
   * @param filename The name of the file
   * @returns The URL to access the file
   */
  public getFileUrl(
    collectionName: string,
    recordId: string,
    filename: string
  ): string {
    const pb = this.auth.getPocketBase();
    const token = pb.authStore.token;
    const url = `${pb.baseUrl}/api/files/${collectionName}/${recordId}/${filename}`;
    return url;
  }

  /**
   * Delete a file from a record
   * @param collectionName The name of the collection
   * @param recordId The ID of the record containing the file
   * @param field The field name of the file to delete
   * @returns The updated record
   */
  public async deleteFile<T = any>(
    collectionName: string,
    recordId: string,
    field: string
  ): Promise<T> {
    if (!this.auth.isAuthenticated()) {
      throw new Error("User must be authenticated to delete files");
    }

    try {
      this.auth.setUpdating(true);
      const pb = this.auth.getPocketBase();
      const data = { [field]: null };
      const result = await pb.collection(collectionName).update<T>(recordId, data);
      return result;
    } catch (err) {
      console.error(`Failed to delete file from ${collectionName}:`, err);
      throw err;
    } finally {
      this.auth.setUpdating(false);
    }
  }

  /**
   * Download a file
   * @param collectionName The name of the collection
   * @param recordId The ID of the record containing the file
   * @param filename The name of the file
   * @returns The file blob
   */
  public async downloadFile(
    collectionName: string,
    recordId: string,
    filename: string
  ): Promise<Blob> {
    if (!this.auth.isAuthenticated()) {
      throw new Error("User must be authenticated to download files");
    }

    try {
      this.auth.setUpdating(true);
      const url = this.getFileUrl(collectionName, recordId, filename);
      const response = await fetch(url);
      
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }
      
      const result = await response.blob();
      return result;
    } catch (err) {
      console.error(`Failed to download file from ${collectionName}:`, err);
      throw err;
    } finally {
      this.auth.setUpdating(false);
    }
  }

  /**
   * Get multiple files from a record
   * @param collectionName The name of the collection
   * @param recordId The ID of the record containing the files
   * @param field The field name containing the files
   * @returns Array of file URLs
   */
  public async getFiles(
    collectionName: string,
    recordId: string,
    field: string
  ): Promise<string[]> {
    if (!this.auth.isAuthenticated()) {
      throw new Error("User must be authenticated to get files");
    }

    try {
      this.auth.setUpdating(true);
      const pb = this.auth.getPocketBase();
      
      // Get the record to retrieve the filenames
      const record = await pb.collection(collectionName).getOne(recordId);
      
      // Get the filenames from the specified field
      const filenames = record[field] || [];
      
      // Convert filenames to URLs
      const fileUrls = filenames.map((filename: string) => 
        this.getFileUrl(collectionName, recordId, filename)
      );
      
      return fileUrls;
    } catch (err) {
      console.error(`Failed to get files from ${collectionName}:`, err);
      throw err;
    } finally {
      this.auth.setUpdating(false);
    }
  }

  /**
   * Compress an image file if it's too large
   * @param file The image file to compress
   * @param maxSizeInMB Maximum size in MB
   * @returns Promise<File> The compressed file
   */
  public async compressImageIfNeeded(file: File, maxSizeInMB: number = 50): Promise<File> {
    if (!file.type.startsWith('image/')) {
      return file;
    }

    const maxSizeInBytes = maxSizeInMB * 1024 * 1024;
    if (file.size <= maxSizeInBytes) {
      return file;
    }

    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.readAsDataURL(file);
      reader.onload = (e) => {
        const img = new Image();
        img.src = e.target?.result as string;
        
        img.onload = () => {
          const canvas = document.createElement('canvas');
          let width = img.width;
          let height = img.height;
          
          // Calculate new dimensions while maintaining aspect ratio
          const maxDimension = 3840; // Higher quality for larger files
          if (width > height && width > maxDimension) {
            height *= maxDimension / width;
            width = maxDimension;
          } else if (height > maxDimension) {
            width *= maxDimension / height;
            height = maxDimension;
          }

          canvas.width = width;
          canvas.height = height;
          
          const ctx = canvas.getContext('2d');
          ctx?.drawImage(img, 0, 0, width, height);
          
          // Convert to blob with higher quality for larger files
          canvas.toBlob(
            (blob) => {
              if (!blob) {
                reject(new Error('Failed to compress image'));
                return;
              }
              resolve(new File([blob], file.name, {
                type: 'image/jpeg',
                lastModified: Date.now(),
              }));
            },
            'image/jpeg',
            0.85 // Higher quality setting for larger files
          );
        };
        
        img.onerror = () => {
          reject(new Error('Failed to load image for compression'));
        };
      };
      
      reader.onerror = () => {
        reject(new Error('Failed to read file for compression'));
      };
    });
  }
} 